==> imsdk_client_flutter-0.0.7.tar.gz: lib/app/session_home.dart <==
import 'package:flutter/material.dart';
import 'package:flutter_slidable/flutter_slidable.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';

import 'app_config.dart';
import 'app_locale.dart';
import 'app_router.dart';
import 'widget/widget_avator.dart';
import 'widget/widget_badge.dart';
import 'widget/widget_divider.dart';

///
/// 会话列表页面
///
class SessionHome extends StatefulWidget {
  const SessionHome({Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _SessionHomeState();
}

class _SessionHomeState extends State<SessionHome> {
  @override
  void initState() {
    super.initState();
    AppConfig.logDebug(['SessionHome.initState()']);
    AppConfig.client.setOnUserFetchedWatcher(_clientCommonListener, remove: false);
    AppConfig.client.setOnTeamFetchedWatcher(_clientCommonListener, remove: false);
    AppConfig.client.setOnUserUpdateListener(_clientCommonListener, remove: false);
    AppConfig.client.setOnTeamUpdateListener(_clientCommonListener, remove: false);
    AppConfig.client.setOnWaitShipUpdateListener(_clientCommonListener, remove: false);
    AppConfig.client.setOnUserShipUpdateListener(_clientCommonListener, remove: false);
    AppConfig.client.setOnTeamShipUpdateListener(_clientCommonListener, remove: false);
    AppConfig.client.setOnMessageSendListener(_clientCommonListener, remove: false);
  }

  @override
  void dispose() {
    AppConfig.logDebug(['SessionHome.dispose()']);
    AppConfig.client.setOnUserFetchedWatcher(_clientCommonListener, remove: true);
    AppConfig.client.setOnTeamFetchedWatcher(_clientCommonListener, remove: true);
    AppConfig.client.setOnUserUpdateListener(_clientCommonListener, remove: true);
    AppConfig.client.setOnTeamUpdateListener(_clientCommonListener, remove: true);
    AppConfig.client.setOnWaitShipUpdateListener(_clientCommonListener, remove: true);
    AppConfig.client.setOnUserShipUpdateListener(_clientCommonListener, remove: true);
    AppConfig.client.setOnTeamShipUpdateListener(_clientCommonListener, remove: true);
    AppConfig.client.setOnMessageSendListener(_clientCommonListener, remove: true);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    return Scaffold(
      backgroundColor: themeData.cardColor,
      appBar: _buildAppBar(themeData),
      body: Column(
        children: [
          WidgetDivider.forFull,
          Expanded(child: _buildBody(themeData)),
          WidgetDivider.forFull,
        ],
      ),
    );
  }

  AppBar _buildAppBar(ThemeData themeData) {
    return AppBar(
      leading: IconButton(
        icon: WidgetBadge(icon: const Icon(Icons.perm_contact_calendar_outlined), count: AppConfig.client.waitshipState.unread),
        tooltip: AppLocalizations.sessionHomeActionBtnContactTip,
        onPressed: () => AppRouter.pushContactHomePage(context),
      ),
      title: Text(AppLocalizations.appHomeTabItemSession),
      actions: [
        IconButton(
          icon: const Icon(Icons.qr_code_scanner_outlined, size: AppConfig.bigGlobalIconReSize),
          tooltip: AppLocalizations.sessionHomeActionBtnQrScanTip,
          onPressed: () => AppRouter.pushQrcodeScanerPage(context, callback: _onQrScanCallback),
        ),
      ],
    );
  }

  Widget _buildBody(ThemeData themeData) {
    return ListView.builder(
      physics: AppConfig.bodyGlobalScrollPhysics,
      itemCount: AppConfig.client.sessionState.okList.length,
      itemBuilder: (context, index) {
        final item = AppConfig.client.sessionState.okList[index];
        if (item is UserShip) {
          return Column(
            children: [
              Slidable(
                endActionPane: ActionPane(
                  motion: AppConfig.slidableScrollMotion,
                  children: [
                    SlidableAction(backgroundColor: Colors.blueAccent, foregroundColor: Colors.white, label: AppLocalizations.sessionHomeListItemBtnReaded, onPressed: (context) => _readedSession(item)),
                    SlidableAction(backgroundColor: Colors.deepOrange, foregroundColor: Colors.white, label: AppLocalizations.sessionHomeListItemBtnRemove, onPressed: (context) => _removeSession(item)),
                  ],
                ),
                child: ListTile(
                  dense: true,
                  tileColor: item.top ? themeData.scaffoldBackgroundColor : null,
                  leading: WidgetAvator(type: WidgetAvatorType.middle, url: item.displayIcon, urlList: item.displayHead, onFormatUrl: AppConfig.formatAttachUrl),
                  title: Row(
                    children: [
                      Expanded(child: Text(item.displayNick, style: themeData.textTheme.subtitle1, maxLines: 1, overflow: TextOverflow.ellipsis)),
                      Text(ComTools.formatDateTime(item.update), style: themeData.textTheme.caption),
                    ],
                  ),
                  subtitle: Row(
                    children: [
                      Expanded(child: Text(item.recent, style: themeData.textTheme.caption, maxLines: 1, overflow: TextOverflow.ellipsis)),
                      WidgetBadge(align: WidgetBadgeAlign.center, count: item.unread, darkMode: !item.notice),
                    ],
                  ),
                  onTap: () => AppRouter.pushSessionChatPage(context, session: AppConfig.client.createSession(usership: item)),
                ),
              ),
              WidgetDivider.forListTileLeadingAvatorMiddle,
            ],
          );
        } else if (item is TeamShip) {
          return Column(
            children: [
              Slidable(
                endActionPane: ActionPane(
                  motion: AppConfig.slidableScrollMotion,
                  children: [
                    SlidableAction(backgroundColor: Colors.blueAccent, foregroundColor: Colors.white, label: AppLocalizations.sessionHomeListItemBtnReaded, onPressed: (context) => _readedSession(item)),
                    SlidableAction(backgroundColor: Colors.deepOrange, foregroundColor: Colors.white, label: AppLocalizations.sessionHomeListItemBtnRemove, onPressed: (context) => _removeSession(item)),
                  ],
                ),
                child: ListTile(
                  dense: true,
                  tileColor: item.top ? themeData.scaffoldBackgroundColor : null,
                  leading: WidgetAvator(type: WidgetAvatorType.middle, single: ComTools.isTeamShipFromSystem(item), url: item.displayIcon, urlList: item.displayHead, onFormatUrl: AppConfig.formatAttachUrl),
                  title: Row(
                    children: [
                      Expanded(child: Text(item.displayNick, style: themeData.textTheme.subtitle1, maxLines: 1, overflow: TextOverflow.ellipsis)),
                      Text(ComTools.formatDateTime(item.update), style: themeData.textTheme.caption),
                    ],
                  ),
                  subtitle: Row(
                    children: [
                      Expanded(child: Text(item.recent, style: themeData.textTheme.caption, maxLines: 1, overflow: TextOverflow.ellipsis)),
                      WidgetBadge(align: WidgetBadgeAlign.center, count: item.unread, darkMode: !item.notice),
                    ],
                  ),
                  onTap: () => AppRouter.pushSessionChatPage(context, session: AppConfig.client.createSession(teamship: item)),
                ),
              ),
              WidgetDivider.forListTileLeadingAvatorMiddle,
            ],
          );
        } else {
          return const SizedBox();
        }
      },
    );
  }

  bool _onQrScanCallback(BuildContext context, String result) {
    return AppRouter.pushSpecifiedUrlPage(context, url: result, pushReplacement: true);
  }

  void _readedSession(Object item) {
    if (item is UserShip) {
      final ship = UserShipDirty();
      ship.unread = 0;
      AppConfig.client.userShipUpdate(id: item.id, fields: ship.data);
    } else if (item is TeamShip) {
      final ship = TeamShipDirty();
      ship.unread = 0;
      AppConfig.client.teamShipUpdate(id: item.id, fields: ship.data);
    }
  }

  void _removeSession(Object item) {
    if (item is UserShip) {
      final ship = UserShipDirty();
      ship.dialog = false;
      AppConfig.client.userShipUpdate(id: item.id, fields: ship.data);
    } else if (item is TeamShip) {
      final ship = TeamShipDirty();
      ship.dialog = false;
      AppConfig.client.teamShipUpdate(id: item.id, fields: ship.data);
    }
  }

  void _clientCommonListener(EasyPacket packet) {
    if (mounted) {
      setState(() {});
    }
  }
}

==> imsdk_client_flutter-0.0.11.tar.gz: lib/app/session_home.dart <==
import 'package:flutter/material.dart';
import 'package:flutter_slidable/flutter_slidable.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';

import 'app_config.dart';
import 'app_locale.dart';
import 'app_router.dart';
import 'widget/widget_avator.dart';
import 'widget/widget_badge.dart';
import 'widget/widget_divider.dart';

///
/// 会话列表页面
///
class SessionHome extends StatefulWidget {
  const SessionHome({Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _SessionHomeState();
}

class _SessionHomeState extends State<SessionHome> {
  @override
  void initState() {
    super.initState();
    AppConfig.logDebug(['SessionHome.initState()']);
    AppConfig.client.setOnUserFetchedWatcher(_clientCommonListener, remove: false);
    AppConfig.client.setOnTeamFetchedWatcher(_clientCommonListener, remove: false);
    AppConfig.client.setOnUserUpdateListener(_clientCommonListener, remove: false);
    AppConfig.client.setOnTeamUpdateListener(_clientCommonListener, remove: false);
    AppConfig.client.setOnWaitShipUpdateListener(_clientCommonListener, remove: false);
    AppConfig.client.setOnUserShipUpdateListener(_clientCommonListener, remove: false);
    AppConfig.client.setOnTeamShipUpdateListener(_clientCommonListener, remove: false);
    AppConfig.client.setOnMessageSendListener(_clientCommonListener, remove: false);
  }

  @override
  void dispose() {
    AppConfig.logDebug(['SessionHome.dispose()']);
    AppConfig.client.setOnUserFetchedWatcher(_clientCommonListener, remove: true);
    AppConfig.client.setOnTeamFetchedWatcher(_clientCommonListener, remove: true);
    AppConfig.client.setOnUserUpdateListener(_clientCommonListener, remove: true);
    AppConfig.client.setOnTeamUpdateListener(_clientCommonListener, remove: true);
    AppConfig.client.setOnWaitShipUpdateListener(_clientCommonListener, remove: true);
    AppConfig.client.setOnUserShipUpdateListener(_clientCommonListener, remove: true);
    AppConfig.client.setOnTeamShipUpdateListener(_clientCommonListener, remove: true);
    AppConfig.client.setOnMessageSendListener(_clientCommonListener, remove: true);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    return Scaffold(
      backgroundColor: themeData.cardColor,
      appBar: _buildAppBar(themeData),
      body: Column(
        children: [
          WidgetDivider.forFull,
          Expanded(child: _buildBody(themeData)),
          WidgetDivider.forFull,
        ],
      ),
    );
  }

  AppBar _buildAppBar(ThemeData themeData) {
    return AppBar(
      leading: IconButton(
        icon: WidgetBadge(icon: const Icon(Icons.perm_contact_calendar_outlined), count: AppConfig.client.waitshipState.unread),
        tooltip: AppLocalizations.sessionHomeActionBtnContactTip,
        onPressed: () => AppRouter.pushContactHomePage(context),
      ),
      title: Text(AppLocalizations.appHomeTabItemSession),
      actions: [
        IconButton(
          icon: const Icon(Icons.qr_code_scanner_outlined, size: AppConfig.iconGlobalSizeMedium),
          tooltip: AppLocalizations.sessionHomeActionBtnQrScanTip,
          onPressed: () => AppRouter.pushQrcodeScanerPage(context, callback: _onQrScanCallback),
        ),
      ],
    );
  }

  Widget _buildBody(ThemeData themeData) {
    return ListView.builder(
      physics: AppConfig.globalScrollPhysics,
      itemCount: AppConfig.client.sessionState.okList.length,
      itemBuilder: (context, index) {
        final item = AppConfig.client.sessionState.okList[index];
        if (item is UserShip) {
          return Column(
            children: [
              Slidable(
                endActionPane: ActionPane(
                  motion: AppConfig.listGlobalItemMotion,
                  children: [
                    SlidableAction(backgroundColor: Colors.blueAccent, foregroundColor: Colors.white, label: AppLocalizations.sessionHomeListItemBtnReaded, onPressed: (context) => _readedSession(item)),
                    SlidableAction(backgroundColor: Colors.deepOrange, foregroundColor: Colors.white, label: AppLocalizations.sessionHomeListItemBtnRemove, onPressed: (context) => _removeSession(item)),
                  ],
                ),
                child: ListTile(
                  dense: true,
                  tileColor: item.top ? themeData.scaffoldBackgroundColor : null,
                  leading: WidgetAvator(type: WidgetAvatorType.middle, url: item.displayIcon, urlList: item.displayHead, onFormatUrl: AppConfig.formatAttachUrl),
                  title: Row(
                    children: [
                      Expanded(child: Text(item.displayNick, style: themeData.textTheme.subtitle1, maxLines: 1, overflow: TextOverflow.ellipsis)),
                      Text(ComTools.formatDateTime(item.update), style: themeData.textTheme.caption),
                    ],
                  ),
                  subtitle: Row(
                    children: [
                      Expanded(child: Text(item.recent, style: themeData.textTheme.caption, maxLines: 1, overflow: TextOverflow.ellipsis)),
                      WidgetBadge(align: WidgetBadgeAlign.center, count: item.unread, darkMode: !item.notice),
                    ],
                  ),
                  onTap: () => AppRouter.pushSessionChatPage(context, session: AppConfig.client.createSession(usership: item)),
                ),
              ),
              WidgetDivider.forListTileLeadingAvatorMiddle,
            ],
          );
        } else if (item is TeamShip) {
          return Column(
            children: [
              Slidable(
                endActionPane: ActionPane(
                  motion: AppConfig.listGlobalItemMotion,
                  children: [
                    SlidableAction(backgroundColor: Colors.blueAccent, foregroundColor: Colors.white, label: AppLocalizations.sessionHomeListItemBtnReaded, onPressed: (context) => _readedSession(item)),
                    SlidableAction(backgroundColor: Colors.deepOrange, foregroundColor: Colors.white, label: AppLocalizations.sessionHomeListItemBtnRemove, onPressed: (context) => _removeSession(item)),
                  ],
                ),
                child: ListTile(
                  dense: true,
                  tileColor: item.top ? themeData.scaffoldBackgroundColor : null,
                  leading: WidgetAvator(type: WidgetAvatorType.middle, single: ComTools.isTeamShipFromSystem(item), url: item.displayIcon, urlList: item.displayHead, onFormatUrl: AppConfig.formatAttachUrl),
                  title: Row(
                    children: [
                      Expanded(child: Text(item.displayNick, style: themeData.textTheme.subtitle1, maxLines: 1, overflow: TextOverflow.ellipsis)),
                      Text(ComTools.formatDateTime(item.update), style: themeData.textTheme.caption),
                    ],
                  ),
                  subtitle: Row(
                    children: [
                      Expanded(child: Text(item.recent, style: themeData.textTheme.caption, maxLines: 1, overflow: TextOverflow.ellipsis)),
                      WidgetBadge(align: WidgetBadgeAlign.center, count: item.unread, darkMode: !item.notice),
                    ],
                  ),
                  onTap: () => AppRouter.pushSessionChatPage(context, session: AppConfig.client.createSession(teamship: item)),
                ),
              ),
              WidgetDivider.forListTileLeadingAvatorMiddle,
            ],
          );
        } else {
          return const SizedBox();
        }
      },
    );
  }

  bool _onQrScanCallback(BuildContext context, String result) {
    return AppRouter.pushSpecifiedUrlPage(context, url: result, pushReplacement: true);
  }

  void _readedSession(Object item) {
    if (item is UserShip) {
      final ship = UserShipDirty();
      ship.unread = 0;
      AppConfig.client.userShipUpdate(id: item.id, fields: ship.data);
    } else if (item is TeamShip) {
      final ship = TeamShipDirty();
      ship.unread = 0;
      AppConfig.client.teamShipUpdate(id: item.id, fields: ship.data);
    }
  }

  void _removeSession(Object item) {
    if (item is UserShip) {
      final ship = UserShipDirty();
      ship.dialog = false;
      AppConfig.client.userShipUpdate(id: item.id, fields: ship.data);
    } else if (item is TeamShip) {
      final ship = TeamShipDirty();
      ship.dialog = false;
      AppConfig.client.teamShipUpdate(id: item.id, fields: ship.data);
    }
  }

  void _clientCommonListener(EasyPacket packet) {
    if (mounted) {
      setState(() {});
    }
  }
}
